local alpha = false
--[===[@debug@
debug = true
--@end-debug@]===]

local L = LibStub("AceLocale-3.0"):GetLocale("LorewalkersHelper")

LorewalkersHelper = LibStub("AceAddon-3.0"):NewAddon("LorewalkersHelper", "AceConsole-3.0")

local lmd = LibStub("LibMapData-1.0")

-- locations of every achievement criteria
local achiMap = {
  {7230, {p={
    {m=857, x=81.5, y=11.5, id=20415},
    {m=806, x=37.3, y=30.1, id=20416},
    {m=809, x=44.7, y=52.4, id=20417}
  }}},
  {6856, {p={
    {m=807, x=20.3, y=55.9, id=19793},
    {m=807, x=55.0, y=47.3, id=19794},
    {m=807, x=34.6, y=64.0, id=19795},
    {m=857, x=72.2, y=31.1, id=19796}
  }}},
  {6716, {p={
    {m=806, x=67.8, y=29.3, id=19618},
    {m=873, x=54.9, y=15.3, id=19619},
    {m=858, x=67.4, y=60.8, id=19620},
    {m=857, x=32.7, y=29.5, id=19621}
  }}},
  {6846, {p={
    {m=806, x=66.0, y=87.5, id=19768},
    {m=807, x=61.2, y=34.6, id=19769},
    {m=857, x=30.6, y=38.5, id=19771},
    {m=809, x=74.5, y=83.5, id=19772}
  }}},
  {6857, {p={
    {m=858, x=48.5, y=32.8, id=19797},
    {m=858, x=59.9, y=55.1, id=19798},
    {m=858, x=52.5, y=10.1, id=19799},
    {m=858, x=35.6, y=32.6, id=19800}
  }}},
  {6850, {p={
    {m=806, x=26.5, y=28.4, id=19781},
    {m=857, x=52.4, y=87.6, id=19782},
    {m=807, x=83.1, y=21.2, id=19783},
    {m=809, x=45.8, y=62.0, id=19784}
  }}},
  {6754, {p={
    {m=809, x=50.6, y=48.1, id=19662},
    {m=857, x=51.0, y=31.7, id=19663},
    {m=806, x=42.2, y=17.5, id=19664},
    {m=811, x=40.29, y=77.50, id=19665}
  }}},
  {6855, {p={
    {m=806, x=47.1, y=45.1, id=19785},
    {m=809, x=43.8, y=51.2, id=19790},
    {m=806, x=55.8, y=56.8, id=19786},
    {m=857, x=40.5, y=56.6, id=19787},
    {m=810, x=37.7, y=63.0, id=19789},
    {m=809, x=67.8, y=48.4, id=19788},
    {m=809, x=41.0, y=42.4, id=19791},
    {m=811, x=68.85, y=44.21, id=19792}
  }}},
  {6847, {p={
    {m=809, x=50.5, y=79.3, id=19773},
    {m=810, x=65.5, y=50.0, id=19774},
    {m=809, x=71.7, y=63.0, id=19775},
    {m=810, x=84.1, y=72.8, id=19776}
  }}},
  {6858, {p={
    {m=807, x=18.8, y=31.8, id=19801},
    {m=811, x=52.91, y=68.73, id=19802},
    {m=806, x=35.7, y=30.5, id=19803},
    {m=811, x=26.59, y=20.48, id=19804},
    {m=809, x=63.1, y=40.8, id=19809}
  }}}
}

local last_update = 0


function LorewalkersHelper:OnInitialize()
  -- Called when the addon is loaded
  self:RegisterChatCommand("lorewalkershelper", "SlashCommand")
  self:RegisterChatCommand("lwh", "SlashCommand")
end

function LorewalkersHelper:PrintUsage()
  local s

  s = "\n"
  s = s .. "|cff7777ff/lorewalkershelper ...|r\n"
  s = s .. "|cff7777ff/lwh ...|r\n"
  s = s .. "|cff7777ff/lwh missing|r " .. L["add waypoints to missing locations in current zone"] .. "\n"
  s = s .. "|cff7777ff/lwh all missing|r " .. L["waypoints to missing locations in all Pandaria zones"] .. "\n"
  s = s .. "|cff7777ff/lwh all|r " .. L["add waypoints to every location across Pandaria involved with the achievements"] .. "\n"
  s = s .. "|cff7777ff/lwh all zone|r " .. L["add waypoints to every location in current zone involved with the achievements"]

  self:Print(s)
end

function LorewalkersHelper:SlashCommand(command)
  local limitZone, limitMissing
  -- commands: all (everything everywhere), all zone (all in current zone), missing (missing in current zone, default), all missing (missing in every zone)
  if command:match"^%s*all%s*$" then
    limitZone = false
    limitMissing = false
  elseif command:match"^%s*all%s+zone%s*$" then
    limitZone = true
    limitMissing = false
  elseif command:match"^%s*all%s+missing%s*$" then
    limitZone = false
    limitMissing = true
  elseif command:match"^%s*missing%s*$" then
    limitZone = true
    limitMissing = true
  else
    LorewalkersHelper:PrintUsage()
  end

  LorewalkersHelper:SetWaypoints(limitZone, limitMissing)

end

function LorewalkersHelper:SetWaypoints(limitZone, limitMissing)
  -- Set waypoints for current zone
  local i, a, c, zoneName, count, doAdd

  -- check zone: must be in Pandaria
  -- check level: suggest level 90
	-- check zone: strongly reccomend 90 in Veiled Stair ;)

  zoneName = GetZoneText()
  count = 0
  for i,a in pairs(achiMap) do
    for i,c in pairs(a[2].p) do
      doAdd = true
      -- check zone
      if limitZone and zoneName ~= lmd:MapLocalize(c.m) then
        doAdd = false
      end
      -- check complete
      if limitMissing and
        select(3, GetAchievementCriteriaInfoByID(a[1], c.id)) then
        doAdd = false
      end
      if doAdd then
        --@alpha@
        TomTom:AddMFWaypoint(c.m, nil, c.x / 100, c.y / 100,
                             {
                              title = select(1, GetAchievementCriteriaInfoByID(a[1], c.id)) .. " (" .. c.id .. ")"
                             }
                          )
        --@end-alpha@
        --[===[@non-alpha@
        TomTom:AddMFWaypoint(c.m, nil, c.x / 100, c.y / 100,
                             {
                              title = select(1, GetAchievementCriteriaInfoByID(a[1], c.id))
                             }
                          )
        --@end-non-alpha@]===]
        count = count + 1
      end
    end
  end
  if limitZone then
    self:Print(L['Added %d waypoints in %s']:format(count, zoneName))
  else
    self:Print(L['Added %d waypoints in %s']:format(count,
                                                  select(6, GetMapContinents())))
  end
  if count > 0 then
    TomTom:SetClosestWaypoint()
  end
end

function LorewalkersHelper:OnEnable()
  -- Called when the addon is enabled
  if not self.updateFrame then
    self.updateFrame = CreateFrame("Frame")
    self.updateFrame:SetScript("OnUpdate", function(frame, elapsed)
      last_update = last_update + elapsed
      if last_update > 2 then
        -- decide whether to show image frame
        last_update = 0
      else
        -- Display:UpdateIconPositions()
      end
    end)
  end
	self.updateFrame:Show()
end

function LorewalkersHelper:OnDisable()
    -- Called when the addon is disabled
  if self.updateFrame then
    self.updateFrame:Hide()
  end
end


-- LDB

local LDBIcon = LDB and LibStub("LibDBIcon-1.0", true)

-- local LibQTip = LibStub('LibQTip-1.0')
local LDB = LibStub("LibDataBroker-1.1"):NewDataObject("LorewalkersHelper",
{
  type = "data source",
  label = "Lorewalkers Helper",
  text = "Lorewalkers Helper",
  -- icon = "Interface\\Icons\\Ability_mount_cloudmount",
  icon = "Interface\\Icons\\achievement_faction_lorewalkers",
  OnClick = function(clickedFrame, button)
    -- Click to add waypoints to missing criteria in current zone. (default)
    -- Shift-Click to add waypoints to missing criteria in Pandaria.
    -- Alt-Click to add waypoints to all criteria in current zone.
    -- Alt-Shift-Click to add waypoints to all criteria in Pandaria.
    if button == "LeftButton" then
      -- LorewalkersHelper:SetWaypoints(limitZone, limitMissing)
      LorewalkersHelper:SetWaypoints(not IsShiftKeyDown(), not IsAltKeyDown())
    end
  end,
});

function LDB:OnTooltipShow()
  local zoneName, zoneId, counts, i, a, c
  self:AddLine("Lorewalkers Helper")

  self:AddLine(L["Missing achievements criteria in current zone"])

  zoneId = -1
  zoneName = GetZoneText()
  counts = {}
  for i, a in pairs(achiMap) do
    for i, c in pairs(a[2].p) do
      if not select(3, GetAchievementCriteriaInfoByID(a[1], c.id)) then
        if counts[c.m] then
          counts[c.m] = counts[c.m] + 1
        else
          counts[c.m] = 1
        end
        if zoneName == lmd:MapLocalize(c.m) then
          zoneId = c.m -- store it for later use ^^
          self:AddLine("|cffff0000" ..
                       select(1, GetAchievementCriteriaInfoByID(a[1], c.id)) ..
                       "|r")
        end
      --@alpha@
      else
        if zoneName == lmd:MapLocalize(c.m) then
          zoneId = c.m -- store it for later use ^^
          self:AddLine("|cff00ff00" ..
                       select(1, GetAchievementCriteriaInfoByID(a[1], c.id)) ..
                       "|r" ..
                       " (" .. a[1] .. " - " .. c.id .. ")")
        end
      --@end-alpha@
      end
    end
  end

  if zoneId >= 0 and not counts[zoneId]  then
    self:AddLine(L["Nothing missing in current zone!"])
  end

  self:AddLine(" ")
  self:AddLine(L["Missing criteria in other zones"])

  for i, c in pairs(counts) do
    if i ~= zoneId then
      if not c then
        self:AddLine(lmd:MapLocalize(i) ..
                     ": |cff00ff00" ..
                     c ..
                     "|r")
      else
        self:AddLine(lmd:MapLocalize(i) ..
                     ": |cffff0000" ..
                     c ..
                     "|r")
      end
    end
  end

  self:AddLine(" ")
  -- colors are Alpha Red Green Blue
  self:AddLine(L["|cffed55aaClick|r to add waypoints to missing criteria in current zone."])
  self:AddLine(L["|cffed55aaShift-Click|r to add waypoints to missing criteria in Pandaria."])
  self:AddLine(L["|cffed55aaAlt-Click|r to add waypoints to all criteria in current zone."])
  self:AddLine(L["|cffed55aaAlt-Shift-Click|r to add waypoints to all criteria in Pandaria."])


  --[===[@debug@
  -- for i, a in pairs(achiMap) do
  --   for i, c in pairs(a[2].p) do
  --     self:AddLine(a[1] .. "." .. c.id .. " (" .. c.m .. ": " .. c.x .. ", " .. c.y .. "): " ..
  --                  "|cffff0000" ..
  --                  select(1, GetAchievementCriteriaInfoByID(a[1], c.id)) ..
  --                  "|r")
  --   end
  -- end
  --@end-debug@]===]
  --[===[@non-debug@
  --@non-debug@]===]

end

function LDB:OnEnter()
  GameTooltip:SetOwner(self, "ANCHOR_NONE")
  GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
  GameTooltip:ClearLines()
  LDB.OnTooltipShow(GameTooltip)
  GameTooltip:Show()
end

function LDB:OnLeave()
  GameTooltip:Hide()
end
